local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local FORCE_DEBUG = false;

if LunaEclipse_SpecManager:isCompatableWOWVersion() then
    local moduleName = "EquipManager";
    local EquipManager = LunaEclipse_SpecManager:NewModule(moduleName, "AceConsole-3.0", "AceEvent-3.0");
    addonTable.EquipManager = EquipManager;

    local equipSuccess = true;

	local function stripIconPath(icon)
		local returnValue = icon;

		if icon then
			local iconString = string.upper(icon);

			if string.find(iconString, "INTERFACE\\ICONS\\") then
				returnValue = string.sub(icon, string.len("INTERFACE\\ICONS\\") + 1, -1);
			end
		end

		return returnValue;	
	end

	local function EquipmentSet_Equipped(specInfo)
        local icon, setID, isEquipped = GetEquipmentSetInfoByName(specInfo.name);
		
        return (setID and isEquipped) or false;	
    end

    local function EquipmentSet_Exists(specInfo)
        local icon, setID = GetEquipmentSetInfoByName(specInfo.name);

        return (setID ~= nil) or false;
    end

    function EquipManager:ChatSpamFilter(event, msg, ...)
        if string.find(msg, string.gsub(ERR_LEARN_ABILITY_S:gsub("%.", "%."), "%%s", "(.*)"))
            or string.find(msg, string.gsub(ERR_LEARN_SPELL_S:gsub("%.", "%."), "%%s", "(.*)"))
            or string.find(msg, string.gsub(ERR_SPELL_UNLEARNED_S:gsub("%.", "%."), "%%s", "(.*)"))
            or string.find(msg, string.gsub(ERR_LEARN_PASSIVE_S:gsub("%.", "%."), "%%s", "(.*)")) then
                return true;
        else
            return false, msg, ...;
        end
    end

    function EquipManager:Load(event, specInfo)
        local profile = addonTable.addonSettings.profile;
		
        if specInfo and (profile and profile.AddonSettings.enableEquipManager) and (event ~= addonTable.LESM_LEFT_COMBAT or (event == addonTable.LESM_LEFT_COMBAT and equipSuccess == false)) then
            if EquipmentSet_Exists(specInfo) and not EquipmentSet_Equipped(specInfo) then
                equipSuccess = UseEquipmentSet(specInfo.name);
            elseif not EquipmentSet_Exists(specInfo) then
                EquipManager:Save(event, specInfo);
            end
        end
    end

    function EquipManager:Save(event, specInfo)
        local profile = addonTable.addonSettings.profile;
		
        if specInfo and (profile and profile.AddonSettings.enableEquipManager) then
			SaveEquipmentSet(specInfo.name, LibIconPath_getIDByName(stripIconPath(specInfo.icon)));
        end
    end

    function EquipManager:OnEnable()
        self:RegisterMessage(addonTable.LESM_LOAD, "Save");
        self:RegisterMessage(addonTable.LESM_SPEC_ADDED, "Load");
        self:RegisterMessage(addonTable.LESM_SPEC_CHANGE_START, "Save");
        self:RegisterMessage(addonTable.LESM_SPEC_CHANGE_FINISH, "Load");
        self:RegisterMessage(addonTable.LESM_SPEC_REMOVED, "Save");
        self:RegisterMessage(addonTable.LESM_LEFT_COMBAT, "Load");

        ChatFrame_AddMessageEventFilter("CHAT_MSG_SYSTEM", EquipManager.ChatSpamFilter);
    end

    function EquipManager:OnDisable()
        ChatFrame_RemoveMessageEventFilter("CHAT_MSG_SYSTEM", EquipManager.ChatSpamFilter);

        self:UnregisterMessage(addonTable.LESM_LOAD);
        self:UnregisterMessage(addonTable.LESM_SPEC_ADDED);
        self:UnregisterMessage(addonTable.LESM_SPEC_CHANGE_START);
        self:UnregisterMessage(addonTable.LESM_SPEC_CHANGE_FINISH);
        self:UnregisterMessage(addonTable.LESM_SPEC_REMOVED);
        self:UnregisterMessage(addonTable.LESM_LEFT_COMBAT);
    end
end