local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local FORCE_DEBUG = false;

local baseDataBroker = LibStub("LibDataBroker-1.1");
local addonIcon = LibStub("LibDBIcon-1.0");
local addonTooltip = LibStub("LibQTip-1.0");

if LunaEclipse_SpecManager:isCompatableWOWVersion() and baseDataBroker and addonIcon and addonTooltip then
    local moduleName = "LDBManager";
    local LDBManager = LunaEclipse_SpecManager:NewModule(moduleName, "AceConsole-3.0", "AceEvent-3.0");
    addonTable.LDBManager = LDBManager;

    local dataTextName = "LunaEclipse: Spec Manager";
    local activeSpecializationMenuName = "LunaEclipse_ActiveSpecialization_Menu";
    local lootSpecializationMenuName = "LunaEclipse_LootSpecialization_Menu";
	
    local reloadRequired;
	
    local iconTooltip = nil;
    local dataBroker = nil;

    local activeSpecMenuFrame = CreateFrame("Frame", activeSpecializationMenuName, UIParent, "UIDropDownMenuTemplate");
    local activeSpecList = {
        { text = SPECIALIZATION, isTitle = true, notCheckable = true },
        { notCheckable = true },
        { notCheckable = true },
        { notCheckable = true },
        { notCheckable = true },
    };

    local lootSpecMenuFrame = CreateFrame("Frame", lootSpecializationMenuName, UIParent, "UIDropDownMenuTemplate");
    local lootSpecList = {
        { text = SELECT_LOOT_SPECIALIZATION, isTitle = true, notCheckable = true },
        { notCheckable = true, func = function() SetLootSpecialization(0) end },
        { notCheckable = true },
        { notCheckable = true },
        { notCheckable = true },
        { notCheckable = true },
    };
	
	local LDBDefaults = {
        icon = "Interface\\ICONS\\spell_nature_bloodlust",
        hide = false,
    };
	
    local clickTexture = {
        texture = "Interface\\TUTORIALFRAME\\UI-TUTORIAL-FRAME",
        height = "16", width = "12",
        xOffset = "0", yOffset = "-1",
        xDimension = "512",	yDimension = "512",
        leftClick = {
            left = "9",	right = "69", top = "228", bottom = "308",
        },
        rightClick = {
            left = "9",	right = "69", top = "330", bottom = "410",
        },
    };

    local activeString = string.join("", addonTable.TEXT_COLOR_TAG_ACTIVE, ACTIVE_PETS, addonTable.TEXT_COLOR_TAG_END);
    local inactiveString = string.join("", addonTable.TEXT_COLOR_TAG_INACTIVE, FACTION_INACTIVE, addonTable.TEXT_COLOR_TAG_END);	
    local leftButtonTextureString = string.join("", "|T", clickTexture.texture, ":", string.join(":", clickTexture.height, clickTexture.width, clickTexture.xOffset, clickTexture.yOffset, clickTexture.xDimension, clickTexture.yDimension, clickTexture.leftClick.left, clickTexture.leftClick.right, clickTexture.leftClick.top, clickTexture.leftClick.bottom), "|t");
    local rightButtonTextureString = string.join("", "|T", clickTexture.texture, ":", string.join(":", clickTexture.height, clickTexture.width, clickTexture.xOffset, clickTexture.yOffset, clickTexture.xDimension, clickTexture.yDimension, clickTexture.rightClick.left, clickTexture.rightClick.right, clickTexture.rightClick.top, clickTexture.rightClick.bottom), "|t");

    StaticPopupDialogs["LESM_CONFIRM_RELOAD_UI"] = {
        text = string.join("", "Please confirm you wish to Reload UI.", "\n\n", "This ensures that your profile currently loaded in memory is written to your hard disk!", "\n\n"),
        button1	= "Yes",
        button2	= "No",
        OnAccept = function(self) 
            ReloadUI(); 
        end,
        timeout	= 10,
        whileDead = true,
        hideOnEscape = true,
    };

    StaticPopupDialogs["LESM_TOGGLE_MINIMAP"] = {
        text = string.join("", "A Reload UI is required for the icon to become visible when using MiniMap icon manager addons.", "\n\n", "Do you wish to reload now?", "\n\n"),
        button1	= "Yes",
        button2	= "No",
        OnAccept = function(self) 
            ReloadUI(); 
        end,
        timeout	= 15,
        whileDead = true,
        hideOnEscape = true,
    };

    local function setupTooltip()
        local addonInfo = LunaEclipse_SpecManager:AddonInfo(true);
        local activeSpecGroup = GetSpecialization();
        local currentSpecInfo = addonTable.currentSpecInfo;
        local lootSpecialization = GetLootSpecialization();
        local lootSpecInfo = addonTable.lootSpecInfo;

        iconTooltip:Clear();
        iconTooltip:AddHeader(string.format("%s%s:%s %s%s%s", HIGHLIGHT_FONT_COLOR_CODE, addonInfo.title, addonTable.TEXT_COLOR_TAG_END, addonTable.TEXT_COLOR_TAG_DEFAULT, addonInfo.version, addonTable.TEXT_COLOR_TAG_END));
        iconTooltip:AddLine(" ");

        for counter = 1, 4 do
			local specID, specName = GetSpecializationInfo(counter);
            local specInfo = LunaEclipse_SpecManager:GetSpecInfo(counter);

            if specID then
				iconTooltip:AddLine(string.format("%s%s:%s %s%s%s", HIGHLIGHT_FONT_COLOR_CODE, (specInfo and specName) or "Not Selected", addonTable.TEXT_COLOR_TAG_END, addonTable.TEXT_COLOR_TAG_DEFAULT, (counter == activeSpecGroup and activeString) or inactiveString, addonTable.TEXT_COLOR_TAG_END));
			end
        end

        iconTooltip:AddLine(" ");
        iconTooltip:AddLine(string.format("%s%s:%s %s%s%s", HIGHLIGHT_FONT_COLOR_CODE, SELECT_LOOT_SPECIALIZATION, addonTable.TEXT_COLOR_TAG_END, addonTable.TEXT_COLOR_TAG_DEFAULT, ((lootSpecialization == 0) and string.format(LOOT_SPECIALIZATION_DEFAULT, (lootSpecInfo and lootSpecInfo.name) or "None")) or ((lootSpecInfo and lootSpecInfo.name) or "None"), addonTable.TEXT_COLOR_TAG_END));

        if IsShiftKeyDown() then
            iconTooltip:AddLine(" ");
            iconTooltip:AddLine(string.join("", HIGHLIGHT_FONT_COLOR_CODE, "Mouse Actions using Shift Key", addonTable.TEXT_COLOR_TAG_END));		
            iconTooltip:AddLine(string.join(" ", string.join("", addonTable.TEXT_COLOR_TAG_DEFAULT, "Toggle Minimap icon:", addonTable.TEXT_COLOR_TAG_END), string.join("", "|cff00FF00", "Shift", " + ", addonTable.TEXT_COLOR_TAG_END), leftButtonTextureString));		
            iconTooltip:AddLine(string.join(" ", string.join("", addonTable.TEXT_COLOR_TAG_DEFAULT, "Reload UI:", addonTable.TEXT_COLOR_TAG_END), string.join("", "|cff00FF00", "Shift", " + ", addonTable.TEXT_COLOR_TAG_END), rightButtonTextureString));		
        else
            iconTooltip:AddLine(" ");
            iconTooltip:AddLine(string.join("", HIGHLIGHT_FONT_COLOR_CODE, "Mouse Actions", addonTable.TEXT_COLOR_TAG_END));		
            iconTooltip:AddLine(string.join(" ", string.join("", addonTable.TEXT_COLOR_TAG_DEFAULT, "Change Talent Specialization:", addonTable.TEXT_COLOR_TAG_END), leftButtonTextureString));
            iconTooltip:AddLine(string.join(" ", string.join("", addonTable.TEXT_COLOR_TAG_DEFAULT, "Change Loot Specialization:", addonTable.TEXT_COLOR_TAG_END), rightButtonTextureString));
        end

        iconTooltip:AddLine();
        iconTooltip:Show();
    end

    local function OnLeftClick()
        if IsShiftKeyDown() then
            LDBManager:ToggleMinimapIcon();
        else
			local currentSpecGroupID = GetSpecialization();

            iconTooltip:Hide()

            for counter = 1, 4 do
                local specID, specName = GetSpecializationInfo(counter);

                if specID then
					activeSpecList[counter + 1].text = string.join("", ((currentSpecGroupID == counter) and addonTable.TEXT_COLOR_TAG_ACTIVE) or addonTable.TEXT_COLOR_TAG_INACTIVE, specName, addonTable.TEXT_COLOR_TAG_END);
                    activeSpecList[counter + 1].func = function() SetSpecialization(counter) end;
                else
                    activeSpecList[counter + 1] = nil;
                end
            end

            EasyMenu(activeSpecList, activeSpecMenuFrame, "cursor", -15, -7, "MENU", 2);
        end
    end
	
    local function OnRightClick()
        if IsShiftKeyDown() then
            StaticPopup_Show("LESM_CONFIRM_RELOAD_UI");
        else
			local lootSpecialization = GetLootSpecialization();
            local currentSpecInfo = addonTable.currentSpecInfo;

            iconTooltip:Hide()
            lootSpecList[2].text = string.join("", ((lootSpecialization == 0) and addonTable.TEXT_COLOR_TAG_ACTIVE) or addonTable.TEXT_COLOR_TAG_INACTIVE, string.format(LOOT_SPECIALIZATION_DEFAULT, (currentSpecInfo and currentSpecInfo.name) or "None"), addonTable.TEXT_COLOR_TAG_END);

            for counter = 1, 4 do
                local specID, specName = GetSpecializationInfo(counter);

                if specID then
                    lootSpecList[counter + 2].text = string.join("", ((lootSpecialization == specID) and addonTable.TEXT_COLOR_TAG_ACTIVE) or addonTable.TEXT_COLOR_TAG_INACTIVE, specName, addonTable.TEXT_COLOR_TAG_END);
                    lootSpecList[counter + 2].func = function() SetLootSpecialization(specID) end;
                else
                    lootSpecList[counter + 2] = nil;
                end
            end

            EasyMenu(lootSpecList, lootSpecMenuFrame, "cursor", -15, -7, "MENU", 2);
        end
    end
	
    local function OnClick(frame, button)
        if button == "LeftButton" then
            OnLeftClick();
        elseif button == "RightButton" then
            OnRightClick();
        end
    end

    local function OnLeave(self)
        addonTooltip:Release(iconTooltip);
        iconTooltip = nil;
    end

    local function OnEnter(self)
        iconTooltip = addonTooltip:Acquire("LESM_ToolTip", 1, "LEFT");
        iconTooltip:SmartAnchorTo(self);
		
        setupTooltip();   
    end

    function LDBManager:UpdateToolTip()
        if iconTooltip then
            setupTooltip();
        end
    end

    function LDBManager:UpdateLDB()
        local profile = addonTable.LDBProfile.profile;

        local currentSpecInfo = addonTable.currentSpecInfo;
        local lootSpecInfo = addonTable.lootSpecInfo;
        local talentText = (currentSpecInfo and string.format('|T%s:14:14:0:0:64:64:4:60:4:60|t', currentSpecInfo.icon)) or "N/A";
        local lootText = (lootSpecInfo and string.format('|T%s:14:14:0:0:64:64:4:60:4:60|t', lootSpecInfo.icon)) or "N/A";

        dataBroker.text = string.format('%s: %s   %s: %s', "Spec", talentText, LOOT, lootText) or LDBDefaults.text;
        dataBroker.icon = (currentSpecInfo and currentSpecInfo.icon) or LDBDefaults.icon;
		
        for key, value in pairs(profile.LDBIconStorage) do
            dataBroker[key] = value;
        end

        addonIcon:Refresh(dataTextName, profile.LDBIconStorage);

        if iconTooltip then
            setupTooltip();
        end
    end

    function LDBManager:ToggleMinimapIcon()
        local profile = addonTable.LDBProfile.profile;

 		if profile.LDBIconStorage.hide then
			addonIcon:Hide(dataTextName);
		else
			addonIcon:Show(dataTextName);
			
			if reloadRequired then
				StaticPopup_Show("LESM_TOGGLE_MINIMAP");
				reloadRequired = false;
			end
		end
   end
	
    function LDBManager:OnInitialize()		
        local profile = addonTable.LDBProfile.profile;			
        local LDBIconData = {
            type = "data source",
            OnClick = OnClick,
            OnLeave = OnLeave,
            OnEnter = OnEnter,
        };

        LDBIconData = LunaEclipse_SpecManager:MergeTables(LDBIconData, LDBDefaults);
        LDBIconData = LunaEclipse_SpecManager:MergeTables(LDBIconData, profile.LDBIconStorage);

        reloadRequired = profile.LDBIconStorage.hide;

        dataBroker = baseDataBroker:NewDataObject(dataTextName, LDBIconData);			
        addonIcon:Register(dataTextName, dataBroker, profile.LDBIconStorage);
    end

    function LDBManager:OnEnable()
        self:RegisterEvent("MODIFIER_STATE_CHANGED", "UpdateToolTip");

        self:RegisterMessage(addonTable.LESM_LOAD, "UpdateLDB");
        self:RegisterMessage(addonTable.LESM_SPEC_ADDED, "UpdateLDB");
        self:RegisterMessage(addonTable.LESM_SPEC_CHANGE_FINISH, "UpdateLDB");
        self:RegisterMessage(addonTable.LESM_SPEC_REMOVED, "UpdateLDB");
        self:RegisterMessage(addonTable.LESM_LOOTSPEC_CHANGED, "UpdateLDB");
        self:RegisterMessage(addonTable.LESM_MINIMAP_TOGGLE, "ToggleMinimapIcon");
    end

    function LDBManager:OnDisable()
        self:UnregisterMessage(addonTable.LESM_LOAD);
        self:UnregisterMessage(addonTable.LESM_SPEC_ADDED);
        self:UnregisterMessage(addonTable.LESM_SPEC_CHANGE_FINISH);
        self:UnregisterMessage(addonTable.LESM_SPEC_REMOVED);
        self:UnregisterMessage(addonTable.LESM_LOOTSPEC_CHANGED);
        self:UnregisterMessage(addonTable.LESM_MINIMAP_TOGGLE);

        self:UnregisterEvent("MODIFIER_STATE_CHANGED");
    end
end