local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local FORCE_DEBUG = false;

if LunaEclipse_SpecManager:isCompatableWOWVersion() then
    local moduleName = "MacroManager";
    local MacroManager = LunaEclipse_SpecManager:NewModule(moduleName, "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0");
    addonTable.MacroManager = MacroManager;
	
	local queueTimer = nil;
    local combatLockdown = false;

    local function Macro_ClearMacros()
        for counter = 138, 121, -1 do 
            DeleteMacro(counter);
        end
    end

    local function Macro_ClearButton(buttonID)
		ClearCursor();
        PickupAction(buttonID);
		ClearCursor();
    end

    local function Macro_RestoreButton(macroID, buttonID)
		ClearCursor();
        PickupMacro(macroID);
        PlaceAction(buttonID);
		ClearCursor();
    end

    local function Macro_CreateMacros(profile)
        for index, keys in pairs(profile) do   
            if profile[index] then
                local macroName = profile[index].Name;
                local macroBody = profile[index].Body;
                local macroTexture = profile[index].Texture;

                if string.find(macroBody, "mod:") 
                    or string.find(macroBody, "modifier")
                    or string.find(macroBody, "#showtooltip") then
                        macroTexture = "INV_MISC_QUESTIONMARK";
                end

                if tonumber(index) >= 121 then
                    CreateMacro(macroName, macroTexture, macroBody, 1);
                end
            end
        end
    end

    local function Macro_GetButtonIDs(macroID)
        local returnValue = {};

        for counter = 1, 120 do
            local actionType, actionID = GetActionInfo(counter);

            if (actionType and actionType == "macro") and (actionID and actionID == macroID) then
                table.insert(returnValue, counter);
            end
        end
		
        return returnValue;
    end

    local function Macro_GetMacro(macroID)
        local returnValue = nil;
        local macroButtonIDs = Macro_GetButtonIDs(macroID);
        local macroName, macroTexture, macroBody = GetMacroInfo(macroID);

        if macroName and macroBody and macroTexture then
            returnValue = {
                name = macroName,
                body = macroBody,
                texture = string.gsub(string.upper(macroTexture or "INV_Misc_Note_01"), "INTERFACE\\ICONS\\", ""),
                buttonIDs = macroButtonIDs,
            };
        end

        return returnValue;
    end

	local function ActionBar_MacroButtons_Count()
        local returnValue = 0;

		for counter = 1, 120 do
            local actionType, actionID = GetActionInfo(counter);

            if (actionType and actionType == "macro") and (actionID and actionID >= 121 and actionID <= 138) then
                returnValue = returnValue + 1;
            end
        end

		return returnValue;
	end

    local function ActionBar_MoveMacros(profile)
        for index, keys in pairs(profile) do   
            if profile[index].ButtonIDs ~= nil then	 
                for _, buttonID in pairs(profile[index].ButtonIDs) do
                    Macro_ClearButton(buttonID);
                    Macro_RestoreButton(tonumber(index), buttonID);
                end
            end
        end
    end
	
    local function MacroProfile_Exists(specInfo)
        local profile = addonTable.addonSettings.profile;
		local returnValue = false;
		
        if (profile and profile.AddonSettings.allowZeroMacros) then
			returnValue = (specInfo and addonTable.macroProfile.profile[specInfo.name] ~= nil) or false; 
		else 
			returnValue	= (specInfo and not LunaEclipse_SpecManager:EmptyTable(addonTable.macroProfile.profile[specInfo.name])) or false;
		end

		return returnValue;
    end

	local function MacroProfile_MacroButtons_Count(profile)
        local returnValue = 0;
		
        for index, keys in pairs(profile) do   
            if profile[index].ButtonIDs ~= nil then	 
                for _, buttonID in pairs(profile[index].ButtonIDs) do
                    returnValue = returnValue + 1;
                end
            end
        end

		return returnValue;
	end

	local function MacroProfile_New()
		local returnValue = {};

		for counter = 121, 138 do 
			local macroInfo = Macro_GetMacro(counter);

			if macroInfo then
				returnValue[tostring(counter)] = { 
					Name = macroInfo.name,
					Body = macroInfo.body,
					Texture = macroInfo.texture,
					ButtonIDs = macroInfo.buttonIDs,
				};
			end
		end

		return returnValue;
	end

	local function MacroProfile_Changed(savedProfile, newProfile)
		return not LunaEclipse_SpecManager:EquateTables(savedProfile, newProfile);
	end

	function MacroManager:LoadQueue(savedProfile)
		if savedProfile then
			ActionBar_MoveMacros(savedProfile);	

			if ActionBar_MacroButtons_Count() ~= MacroProfile_MacroButtons_Count(savedProfile) then
				if not queueTimer then
					queueTimer = self:ScheduleRepeatingTimer("LoadQueue", 0.05, savedProfile);
				end
			else
				if queueTimer then
					self:CancelTimer(queueTimer);
					queueTimer = nil;
				end

				if (IsAddOnLoaded("Blizzard_MacroUI")) then
					MacroFrame_Update();
				end
			end
		end		
	end

    function MacroManager:Load(event, specInfo)
        local profile = addonTable.addonSettings.profile;
		
        if specInfo and (profile and profile.AddonSettings.enableMacroManager) and (event ~= addonTable.LESM_LEFT_COMBAT or (event == addonTable.LESM_LEFT_COMBAT and combatLockdown)) then
			if MacroProfile_Exists(specInfo) then
				local savedProfile = addonTable.macroProfile.profile[specInfo.name];
				local newProfile = MacroProfile_New();

				combatLockdown = LunaEclipse_SpecManager:InCombat();

				if not combatLockdown and MacroProfile_Changed(savedProfile, newProfile) then		
					Macro_ClearMacros();
					Macro_CreateMacros(savedProfile);
					self:LoadQueue(savedProfile);
				end
			else
				self:Save(event, specInfo, true);
			end
        end
    end

    function MacroManager:Save(event, specInfo, forceSave)
        local profile = addonTable.addonSettings.profile;
		
        if specInfo and (profile and profile.AddonSettings.enableMacroManager) then
			local savedProfile = addonTable.macroProfile.profile[specInfo.name];
			local newProfile = MacroProfile_New();

			if (MacroProfile_Changed(savedProfile, newProfile) and ActionBar_MacroButtons_Count() > 0) or forceSave then
				addonTable.macroProfile.profile[specInfo.name] = {};
				addonTable.macroProfile.profile[specInfo.name] = newProfile;
			end
        end
    end

    function MacroManager:OnEnable()
        self:RegisterMessage(addonTable.LESM_LOAD, "Save");
        self:RegisterMessage(addonTable.LESM_SPEC_ADDED, "Load");
        self:RegisterMessage(addonTable.LESM_SPEC_CHANGE_START, "Save");
        self:RegisterMessage(addonTable.LESM_SPEC_CHANGE_FINISH, "Load");
        self:RegisterMessage(addonTable.LESM_SPEC_REMOVED, "Save");
        self:RegisterMessage(addonTable.LESM_LEFT_COMBAT, "Load");
    end

    function MacroManager:OnDisable()
        self:UnregisterMessage(addonTable.LESM_LOAD);
        self:UnregisterMessage(addonTable.LESM_SPEC_ADDED);
        self:UnregisterMessage(addonTable.LESM_SPEC_CHANGE_START);
        self:UnregisterMessage(addonTable.LESM_SPEC_CHANGE_FINISH);
        self:UnregisterMessage(addonTable.LESM_SPEC_REMOVED);
        self:UnregisterMessage(addonTable.LESM_LEFT_COMBAT);
    end
end