local addonName, addonTable = ...; -- Pulls back the Addon-Local Variables and store them locally.
local FORCE_DEBUG = false;

if LunaEclipse_SpecManager:isCompatableWOWVersion() then
    local moduleName = "OptionsManager";
    local OptionsManager = LunaEclipse_SpecManager:NewModule(moduleName, "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0");
    addonTable.OptionsManager = OptionsManager;

	addonTable.Options = {};
	addonTable.addonOptions = {
		type = "group",
		name = "Module Options",
		handler = OptionsManager,
		childGroups = "tab",
		get = "getValue",
		set = "setValue",
		args = {
			miniMap = {
				type = "header",
				name = "Minimap Icon Options",
				order = 10,
			},
			enableMinimapIcon = {
				type = "toggle",
				name = "Hide Minimap Icon",
				desc = "Hide the minimap icon, this should only be used if you are using a LDB Display.",
				arg = "enableMinimapIcon",
				width = "full",
				order = 11,
			},
			equipManager = {
				type = "header",
				name = "Equipment Manager Options",
				order = 20,
			},
			enableEquipManager = {
				type = "toggle",
				name = "Enable Equipment Manager",
				desc = "Enable the Equipment Manager to automatically change equipment sets when changing specializations.",
				arg = "enableEquipManager",
				width = "full",
				order = 21,
			},
			useOverlay = {
				type = "toggle",
				name = "Use the Equipment Set Overlay",
				desc = "This will show missing items or items that will change as a color overlay when selecting an equipment set.",
				arg = "useOverlay",
				width = "full",
				order = 22,
			},
			macroManager = {
				type = "header",
				name = "Macro Manager Options",
				order = 30,
			},
			enableMacroManager = {
				type = "toggle",
				name = "Enable Macro Manager",
				desc = "Enable the Macro Manager to swap saved macros when changing specializations.",
				arg = "enableMacroManager",
				width = "full",
				order = 31,
			},
			allowZeroMacros = {
				type = "toggle",
				name = "Allow Macro Profiles with Zero Macros",
				desc = "This allows a profile to be considered valid when it contains no macros, unchecking this will cause the addon to treat profiles with no macros as not existing.",
				arg = "allowZeroMacros",
				width = "full",
				order = 32,
			},
		},
	};

	function OptionsManager:OnInitialize()
		LibStub("AceConfig-3.0"):RegisterOptionsTable(addonName, addonTable.addonOptions);
		addonTable.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(addonName, "LunaEclipse: SpecManger");
	end

    function OptionsManager:getValue(info)
		local returnValue;

		if info then
			if info.arg == "enableMinimapIcon" then
 				local profile = addonTable.LDBProfile.profile;

 				returnValue = profile.LDBIconStorage.hide;
			else
				local profile = addonTable.addonSettings.profile;

 				returnValue = profile.AddonSettings[info.arg];
			end
		end

		return returnValue;
	end
	
	function OptionsManager:setValue(info, value)
		if info then
			if info.arg == "enableMinimapIcon" then
 				local profile = addonTable.LDBProfile.profile;

 				profile.LDBIconStorage.hide = value;
				
				self:SendMessage(addonTable.LESM_MINIMAP_TOGGLE);
			else
				local profile = addonTable.addonSettings.profile;

 				profile.AddonSettings[info.arg] = value;
			end
		end
	end
end